//!
//!

#[macro_use]
extern crate diesel;
#[macro_use]
extern crate diesel_migrations;

use diesel::{
  r2d2::{ConnectionManager, Pool},
  PgConnection,
};
use std::env;

mod compositions;
mod events;
mod repositories;
mod services;

diesel_migrations::embed_migrations!();

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
  let url_server_broker = match env::var_os("URL_BROKER") {
    Some(v) => v.into_string().unwrap(),
    None => "".to_string(),
  };
  let queue_server_broker = match env::var_os("QUEUE_BROKER") {
    Some(v) => v.into_string().unwrap(),
    None => "".to_string(),
  };

  let url_db = match env::var_os("URL_DATABASE") {
    Some(v) => v.into_string().unwrap(),
    None => "".to_string(),
  };

  let captcha_salt = match env::var_os("CAPTCHA_SALT") {
    Some(v) => v.into_string().unwrap(),
    None => "".to_string(),
  };
  let password_salt = match env::var_os("PASSWORD_SALT") {
    Some(v) => v.into_string().unwrap(),
    None => "".to_string(),
  };

  let manager = ConnectionManager::<PgConnection>::new(url_db);
  let pool = Pool::builder()
    .max_size(4)
    .test_on_check_out(true)
    .build(manager)?;
  let conn = pool.get()?;

  embedded_migrations::run(&conn)?;

  let server_broker_opts = nats::asynk::Options::new();
  let connection_server_broker = server_broker_opts.connect(url_server_broker).await?;

  let bank_service = services::bank::service::Service::new(
    connection_server_broker.clone(),
    queue_server_broker.clone(),
    pool.clone(),
  );
  let identity_service = services::identity::service::Service::new(
    connection_server_broker.clone(),
    queue_server_broker.clone(),
    pool.clone(),
    captcha_salt.clone(),
    password_salt.clone(),
  );
  let member_service = services::member::service::Service::new(
    connection_server_broker.clone(),
    queue_server_broker.clone(),
    pool.clone(),
    password_salt.clone(),
  );
  let member_bank_account_service = services::member_bank_account::service::Service::new(
    connection_server_broker.clone(),
    queue_server_broker.clone(),
    pool.clone(),
    password_salt.clone(),
  );
  let member_bank_deposit_service = services::member_bank_deposit::service::Service::new(
    connection_server_broker.clone(),
    queue_server_broker.clone(),
    pool.clone(),
  );
  let member_bank_withdraw_service = services::member_bank_withdraw::service::Service::new(
    connection_server_broker.clone(),
    queue_server_broker.clone(),
    pool.clone(),
  );
  let member_class_service = services::member_class::service::Service::new(
    connection_server_broker.clone(),
    queue_server_broker.clone(),
    pool.clone(),
  );
  let member_game_setting_service = services::member_game_setting::service::Service::new(
    connection_server_broker.clone(),
    queue_server_broker.clone(),
    pool.clone(),
  );
  let member_level_service = services::member_level::service::Service::new(
    connection_server_broker.clone(),
    queue_server_broker.clone(),
    pool.clone(),
  );
  let member_referrer_service = services::member_referrer::service::Service::new(
    connection_server_broker.clone(),
    queue_server_broker.clone(),
    pool.clone(),
  );
  let member_session_service = services::member_session::service::Service::new(
    connection_server_broker.clone(),
    queue_server_broker.clone(),
    pool.clone(),
  );
  let member_settlement_setting_service =
    services::member_settlement_setting::service::Service::new(
      connection_server_broker.clone(),
      queue_server_broker.clone(),
      pool.clone(),
    );
  let site_service = services::site::service::Service::new(
    connection_server_broker.clone(),
    queue_server_broker.clone(),
    pool.clone(),
  );

  let member_event_handler = events::member::event::EventHandler::new(
    connection_server_broker.clone(),
    queue_server_broker.clone(),
    pool.clone(),
  );

  println!("Server service [beteran-server-service] is started");

  futures::try_join!(
    bank_service.subscribe(),
    identity_service.subscribe(),
    member_service.subscribe(),
    member_bank_account_service.subscribe(),
    member_bank_deposit_service.subscribe(),
    member_bank_withdraw_service.subscribe(),
    member_class_service.subscribe(),
    member_game_setting_service.subscribe(),
    member_level_service.subscribe(),
    member_session_service.subscribe(),
    member_referrer_service.subscribe(),
    member_settlement_setting_service.subscribe(),
    site_service.subscribe(),
    member_event_handler.subscribe(),
  )?;

  Ok(())
}
