//!
//!
use super::{models, schema::roles};
use beteran_common_rust as bcr;
use diesel::prelude::*;
use diesel::result::Error;

///
pub struct Repository {}

impl std::fmt::Debug for Repository {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Repository of roles").finish()
  }
}

impl Default for Repository {
  fn default() -> Self {
    Self::new()
  }
}

impl Repository {
  ///
  pub fn new() -> Repository {
    Repository {}
  }

  ///
  pub fn insert(
    &self,
    conn: &diesel::PgConnection,
    new_resource: &models::NewRole,
  ) -> Result<models::Role, Error> {
    let inserted = diesel::insert_into(roles::table)
      .values(new_resource)
      .get_result::<models::Role>(conn)?;

    Ok(inserted)
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
  ) -> Result<Option<models::Role>, Error> {
    match roles::table
      .find(id as uuid::Uuid)
      .first::<models::Role>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<i64, Error> {
    let mut q = roles::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = s.parent_id {
        q = q.filter(roles::dsl::parent_id.eq(sp));
      }
      if let Some(sp) = &s.name_like {
        q = q.filter(roles::dsl::name.like(sp));
      }
      if let Some(sp) = &s.key_like {
        q = q.filter(roles::dsl::key.like(sp));
      }
      if let Some(sp) = &s.description_like {
        q = q.filter(roles::dsl::description.like(sp));
      }
      if let Some(sp) = s.can_use {
        q = q.filter(roles::dsl::can_use.eq(sp));
      }
    }

    q.count().get_result(conn)
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: models::FindAll,
  ) -> Result<Vec<models::Role>, Error> {
    let mut q = roles::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = s.parent_id {
        q = q.filter(roles::dsl::parent_id.eq(sp));
      }
      if let Some(sp) = &s.name_like {
        q = q.filter(roles::dsl::name.like(sp));
      }
      if let Some(sp) = &s.key_like {
        q = q.filter(roles::dsl::key.like(sp));
      }
      if let Some(sp) = &s.description_like {
        q = q.filter(roles::dsl::description.like(sp));
      }
      if let Some(sp) = s.can_use {
        q = q.filter(roles::dsl::can_use.eq(sp));
      }
    }

    if let Some(p) = &find_all.pagination {
      let page = p.page.unwrap_or(1);

      if let Some(page_size) = p.page_size {
        q = q.offset(((page - 1) * page_size) as i64);
        q = q.limit(page_size as i64);
      }
    }
    if let Some(orderbys) = &find_all.sorts {
      for s in orderbys {
        match s {
          bcr::pagination::Sort::ASC(property) => match property.as_str() {
            "name" => {
              q = q.order_by(roles::name.asc());
            }
            "key" => {
              q = q.order_by(roles::key.asc());
            }
            "can_use" => {
              q = q.order_by(roles::can_use.asc());
            }
            "created_at" => {
              q = q.order_by(roles::created_at.asc());
            }
            "updated_at" => {
              q = q.order_by(roles::updated_at.asc());
            }
            _ => {}
          },
          bcr::pagination::Sort::DESC(property) => match property.as_str() {
            "name" => {
              q = q.order_by(roles::name.desc());
            }
            "key" => {
              q = q.order_by(roles::key.desc());
            }
            "can_use" => {
              q = q.order_by(roles::can_use.desc());
            }
            "created_at" => {
              q = q.order_by(roles::created_at.desc());
            }
            "updated_at" => {
              q = q.order_by(roles::updated_at.desc());
            }

            _ => {}
          },
        };
      }
    }

    q.load::<models::Role>(conn)
  }

  ///
  pub fn update(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
    modify: &models::ModifyRole,
  ) -> Result<u64, Error> {
    use roles::dsl;

    diesel::update(dsl::roles.filter(dsl::id.eq(id)))
      .set(modify)
      .execute(conn)
      .map(|c| c as u64)
  }

  ///
  pub fn update_can_use(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
    modify: &models::ModifyRole4CanUse,
  ) -> Result<u64, Error> {
    use roles::dsl;

    diesel::update(dsl::roles.filter(dsl::id.eq(id)))
      .set(modify)
      .execute(conn)
      .map(|c| c as u64)
  }

  ///
  pub fn delete(&self, conn: &diesel::PgConnection, id: uuid::Uuid) -> Result<u64, Error> {
    use roles::dsl;

    diesel::delete(roles::table.filter(dsl::id.eq(id)))
      .execute(conn)
      .map(|c| c as u64)
  }
}
